@@ -48,6 +48,9 @@ def itemgetter(*args,**kwargs):
             return lambda x:(x[args[0]],)
     return operator.itemgetter(*args)
 
+class CriticalError(Exception):
+    """Critical error"""
+
 class DataVarsError(Exception):
     """Exception of getting variable values"""
 
@@ -145,6 +148,8 @@ class Variable:
         self.invalid = True # self.value is None or self.name in self.parent.iniCache
         self.name = self.getVariableName()
         self.processInvalidate = False
+        # производится чтение из ini
+        self.bIniRead = False
         self.init()
         for key,val in kwargs.items():
             setattr(self,key,val)
@@ -259,20 +264,26 @@ class Variable:
         """
         Read from ini file
         """
-        value = self.parent.iniCache[self.name]['value']
-        section = self.parent.iniCache[self.name]['section']
-        value = self.parent.unserialize(self.type,
-                                 value.encode('utf-8'))
-        if self.mode == READONLY:
-            raise VariableError(
-                _("Attempt to rewrite readonly variable %s")%
-                self.name)
-        else:
-            try:
-                self._set(value)
-            except Exception as e:
-                return False
-        return True
+        if self.bIniRead:
+            return False
+        self.bIniRead = True
+        try:
+            value = self.parent.iniCache[self.name]['value']
+            section = self.parent.iniCache[self.name]['section']
+            value = self.parent.unserialize(self.type,
+                                     value.encode('utf-8'))
+            if self.mode == READONLY:
+                raise VariableError(
+                    _("Attempt to rewrite readonly variable %s")%
+                    self.name)
+            else:
+                try:
+                    self._set(value)
+                except Exception as e:
+                    return False
+            return True
+        finally:
+            self.bIniRead = False
 
     def _get(self):
         """
@@ -383,13 +394,13 @@ class Variable:
         if "list" in self.type:
             if not type(value) in (list,tuple):
                 raise VariableError(
-                    _("Value for variable '{varname}' may be '{vartype}' only").format(
+                    _("The value for variable '{varname}' may be {vartype} only").format(
                     varname=self.label or self.name,
                     vartype="list"))
-            error = _("Values for variable '{varname}' may be '{vartype}' only")
+            error = _("Values for variable '{varname}' may be {vartype} only")
         else:
             value = repeat(value,1)
-            error = _("Value for variable '{varname}' may be '{vartype}' only")
+            error = _("The value for variable '{varname}' may be {vartype} only")
         if "string" in self.type:
             value, valuecopy = tee(value,2)
             for v in (x for x in valuecopy if not type(x) in (str,unicode)):
@@ -399,7 +410,7 @@ class Variable:
             return
         if "choice" in self.type:
             choiceVal = self.choice()
-            tipChoice = map(lambda x:'"%s"(%s)'%(x[1],x[0]) \
+            tipChoice = map(lambda x:'"%s" [%s]'%(x[1],x[0]) \
                             if type(x) in (list,tuple) else str(x), choiceVal)
             if choiceVal and type(choiceVal[0]) in (tuple,list):
                 choiceVal = [x[0] for x in choiceVal]
@@ -407,10 +418,19 @@ class Variable:
             value = repeat(value,1)
         for recval in value:
             for val in recval:
+                name = self.label or self.name
+                if val == "list" and "choice" in self.type:
+                    if "edit" in self.type:
+                        if "list" in self.type:
+                            error = _("Values for variable '{varname}' may "
+                                      "be {vartype}")
+                        else:
+                            error = _("The value for variable '{varname}' may be "
+                                      "{vartype}")
+                    self.raiseWrongChoice(name, tipChoice, val, error)
                 if "choice" in self.type and not "choiceedit" in self.type:
                     if choiceVal and val and not val in choiceVal or \
                         val is None:
-                        name = self.label or self.name
                         if not choiceVal:
                             self.raiseNothingValue()
                         self.raiseWrongChoice(name,tipChoice,val,error)
@@ -569,7 +589,7 @@ class TableVariable(Variable):
         if not type(value) in (list,tuple) or \
             any(i for i in value if not type(i) in (tuple,list)):
             raise VariableError(
-                    _("Value for {varname} may be '{vartype}' only").format(
+                    _("The value for {varname} may be '{vartype}' only").format(
                     varname=self.label or self.name,
                     vartype="table"))
         # check len all entries
@@ -762,7 +782,10 @@ class SimpleDataVars:
     def Get(self,varname,humanreadable=False):
         varname = varname.rpartition(".")[2]
         if not varname in self.cache:
-            self.cache[varname] = self.allVars.get(varname).get()
+            if self.allVars.get(varname) is not None:
+                self.cache[varname] = self.allVars.get(varname).get()
+            else:
+                raise DataVarsError(_("Failed to get value %s")%varname)
         return self.cache[varname]
 
     def getInfo(self,varname):
@@ -775,45 +798,58 @@ class SimpleDataVars:
         """
         if path.isdir(iniFile):
             config = cl_template.iniParser(
-                text=reduce(lambda x,y:x+readFile(y),
-                addStdConfig(
-                searchProfile(iniFile,'calculate.env'),prefix=self.systemRoot),""))
+                text=reduce(lambda x, y: x + readFile(y),
+                            addStdConfig(
+                                searchProfile(iniFile, 'calculate.env'),
+                                prefix=self.systemRoot), ""))
         else:
             config = cl_template.iniParser(iniFile)
-        importVars = config.getAreaVars("main")
-        for varname in self.allVars.keys():
+        importVars = {}
+        for varname, varobj in self.allVars.items():
+            if varobj.section not in importVars:
+                importVars[varobj.section] = config.getAreaVars(varobj.section)
             if not varname in self.cache:
-                val = self.unserialize("string",importVars.get(varname,''))
-                if val:
-                    self.cache[varname] = val.encode('utf-8')
+                val = self.unserialize(varobj.type or "string",
+                    importVars[varobj.section].get(varname,'').encode('utf-8'))
+                if val or varname in importVars[varobj.section]:
+                    self.cache[varname] = val
 
-    def serialize(self,varType,value):
+    @staticmethod
+    def serialize(varType, value):
         """
         Serialize to string for ini file
         """
         fixEmpty = lambda x: str(x) or "''"
-        fixSpace = lambda x: "'%s'"%str(x) if " " in str(x) else str(x)
-        isListOrTuple = lambda x: type(x) in (list,tuple)
+        fixSpace = lambda x: "'%s'" % str(x) if " " in str(x) else str(x)
+        isListOrTuple = lambda x: type(x) in (list, tuple)
         if isListOrTuple(value):
             if "list" in varType:
-                return ",".join(map(fixEmpty,value))
+                return ",".join(map(fixEmpty, value))
             elif "table" in varType:
-                return ";".join(map(lambda x:",".join(map(fixEmpty,x)),
-                                value))
+                return ",".join(map(lambda x: ":".join(map(fixEmpty, x)),
+                                    value))
         return fixEmpty(value)
 
-    def unserialize(self,varType,value):
+    @staticmethod
+    def unserialize(varType, value):
         """
         Unserialize form string for varname
         """
-        fixEmpty = lambda x:"" if x=="''" else x
-        def getList(val):
-            return map(fixEmpty,value.split(','))
+        fixEmpty = lambda x: "" if x == "''" or x == '""' else x.strip()
+
+        def getList(delimeter=','):
+            def wrapper(val):
+                if val == "":
+                    return []
+                return map(fixEmpty, val.split(delimeter))
+
+            return wrapper
+
         if "list" in varType:
-            return getList(value)
+            return getList()(value)
         if "table" in varType:
-            return map(getList,value.split(';'))
-        return fixEmpty(value).strip("'")
+            return map(getList(':'), value.split(','))
+        return fixEmpty(value).strip("'").strip('"')
 
 
 class DataVars(SimpleDataVars):
@@ -1005,9 +1041,16 @@ class DataVars(SimpleDataVars):
         """
         if not self.iniCache:
             # get initialized section names
-            if os.path.exists('/etc/make.profile'):
+            try:
+                make_profile = self.Get('main.cl_make_profile')
+            except VariableError as e:
+                action = self.Get('cl_action')
+                if not action or action in ['update_profile']:
+                    return True
+                raise CriticalError(str(e))
+            if os.path.exists(make_profile):
                 profiles = [('profile',
-                            path.join('/etc',os.readlink('/etc/make.profile')))]
+                            path.join('/etc',os.readlink(make_profile)))]
             else:
                 profiles = []
             for iniName,iniFile in profiles+\
@@ -1109,7 +1152,8 @@ class DataVars(SimpleDataVars):
             else:
                 res = varObj._get()
             if not humanreadable:
-                self.requestVariables.pop()
+                if self.requestVariables:
+                    self.requestVariables.pop()
             return res
         except BaseException,e:
             while(len(self.requestVariables)):
@@ -1494,10 +1538,29 @@ class DataVars(SimpleDataVars):
             else:
                 yield w
 
+    def printGroup(self,info):
+        """
+        Вывести группу переменных
+        """
+        print "=DEBUG.SET="
+        for vname in sorted(filter(lambda x:x.lower() == x,
+                          info._type_info.keys())):
+            print "|", vname,"=",getattr(info,vname)
+        print "=DEBUG.DATAVARS="
+        for vname in self.mapVarGroup.keys():
+            print "|", vname, "=", self.Get(vname)
+        print "=DEBUG.END="
+
     def checkGroups(self,info,allvars=False):
         """
         Check variables in group or all in info
         """
+        def compare_choice(part_s, *options):
+            lower_unicode = lambda x: _toUNICODE(x).lower()
+            part_s = lower_unicode(part_s)
+            options = [lower_unicode(x) for x in options]
+            return any(x == part_s for x in options)
+
         if not self.groups:
             return []
         errors = []
@@ -1563,13 +1626,11 @@ class DataVars(SimpleDataVars):
                                 res = []
                                 for val in vals:
                                     if not val in map(lambda x:x[0],choicedata):
-                                        result = filter(lambda x:(
-                                                    _toUNICODE(val).lower() in _toUNICODE(x[0]).lower() or \
-                                                    _toUNICODE(val).lower() in _toUNICODE(x[1]).lower()),
+                                        result = filter(lambda x:compare_choice(val, x[0], x[1]),
                                                  choicedata)
                                         if len(result) > 1:
                                             raise VariableError(_("Ambiguous choice:%s")%
-                                                ",".join(map(
+                                                formatListOr(map(
                                                 lambda x:'"%s"(%s)'%(x[1],x[0]) if x[0] != x[1] else x[0],
                                                              result)))
                                         elif result:
@@ -1598,7 +1659,9 @@ class DataVars(SimpleDataVars):
                             break
                     else:
                         typeError = 'error'
-                    errors.append({'type':typeError, 'field':var, 'message':mess})
+                    errors.append({'type':typeError, 'field':var,
+                                   'field_obj': self.getInfo(var) if var else "",
+                                   'message':mess})
                 except BaseException as e:
                     for i in apply(traceback.format_exception, sys.exc_info()):
                         print i,
